import * as Vite from 'vite';
import { Plugin } from 'postcss';

interface UniAppTailwindPluginOptions {
    /**
     * @desc 需要应用该插件的环境
     */
    apply?: string[];
    /**
     * @desc 应用该插件的环境判断方法
     */
    getShouldApply?: (targets: string[], current: string) => boolean;
    /**
     * @desc space between 元素映射
     */
    spaceBetweenElements?: string[];
    /**
     * @desc divide width 元素映射
     */
    divideWidthElements?: string[];
    /**
     * @desc uni-app 中元素映射
     */
    elementMap?: [string, string[]][];
    /**
     * @desc 特殊符号映射
     */
    characterMap?: [string, string][];
}
declare type Options = Required<UniAppTailwindPluginOptions>;
declare const defaultOptions: {
    readonly apply: string[];
    readonly getShouldApply: (targets: string[], current: string) => boolean;
    readonly spaceBetweenElements: string[];
    readonly divideWidthElements: string[];
    readonly elementMap: [string, string[]][];
    readonly characterMap: [string, string][];
};

declare const Apply: string[];
declare const GetShouldApply: (targets: string[], current: string) => boolean;
declare const CharacterMap: [string, string][];
declare const SpaceBetweenElements: string[];
declare const DivideWidthElements: string[];
declare const ElementMap: [string, string[]][];

declare function postcssTransformSelector(options?: Options): Plugin;
declare namespace postcssTransformSelector {
    var postcss: boolean;
}

declare const isStyleFile: (fileName: string) => boolean;
declare const transformStyle: (source: string, options?: Options) => string;

declare const babelTransformClass: (source: string, options: Options) => string;

declare const isTemplateFile: (fileName: string) => boolean;
declare const classNames: string[];
declare const transformTemplate: (source: string, options?: Options) => string;

declare function UniAppTailwindPlugin(options?: UniAppTailwindPluginOptions): Vite.Plugin;

export { Apply, CharacterMap, DivideWidthElements, ElementMap, GetShouldApply, Options, SpaceBetweenElements, UniAppTailwindPluginOptions, babelTransformClass, classNames, UniAppTailwindPlugin as default, defaultOptions, isStyleFile, isTemplateFile, postcssTransformSelector, transformStyle, transformTemplate };
